#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
    /// Calculate Deviations from recent swing highs and lows to normalize price movement
	/// 
	/// Code Written by Tyler Moore
	/// www.FreeIndicators.Com
	/// 
	/// For modifications to this code or to have your own custom code written
	/// please visit www.MooreTechLLC.com	
    /// </summary>
    [Description("Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com")]
    public class SwingOscillator : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int strength = 5; // Default setting for Strength
        // User defined variables (add any user defined variables below)
		private MIN min;
		private MAX max;
		private double sh = 0, sl = 0;
		private int shBar = 0, slBar = 0;
        #endregion

        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Bar, "Osc"));
            Overlay				= false;
			PlotsConfigurable=false;
        }
		
		protected override void OnStartUp()
		{
			min=MIN(Low,strength);
			max=MAX(High,strength);
		}

        protected override void OnBarUpdate()
        {
			if (CurrentBar<1)
				return;
			
			if (High[0]>max[1])
			{
				sh=High[0];
				shBar=CurrentBar;
			}
			if (Low[0]<min[1])
			{
				sl=Low[0];
				slBar=CurrentBar;
			}
								
			if (shBar>slBar && slBar>0)			
			{
				PlotColors[0][0]=Color.Green;
				Osc.Set(sh-sl);	
			}
			else if (slBar>shBar && shBar>0)
			{
				PlotColors[0][0]=Color.Red;
				Osc.Set(sl-sh);	
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Osc
        {
            get { return Values[0]; }
        }

        [Description("")]
        [GridCategory("Parameters")]
        public int Strength
        {
            get { return strength; }
            set { strength = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SwingOscillator[] cacheSwingOscillator = null;

        private static SwingOscillator checkSwingOscillator = new SwingOscillator();

        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public SwingOscillator SwingOscillator(int strength)
        {
            return SwingOscillator(Input, strength);
        }

        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public SwingOscillator SwingOscillator(Data.IDataSeries input, int strength)
        {
            if (cacheSwingOscillator != null)
                for (int idx = 0; idx < cacheSwingOscillator.Length; idx++)
                    if (cacheSwingOscillator[idx].Strength == strength && cacheSwingOscillator[idx].EqualsInput(input))
                        return cacheSwingOscillator[idx];

            lock (checkSwingOscillator)
            {
                checkSwingOscillator.Strength = strength;
                strength = checkSwingOscillator.Strength;

                if (cacheSwingOscillator != null)
                    for (int idx = 0; idx < cacheSwingOscillator.Length; idx++)
                        if (cacheSwingOscillator[idx].Strength == strength && cacheSwingOscillator[idx].EqualsInput(input))
                            return cacheSwingOscillator[idx];

                SwingOscillator indicator = new SwingOscillator();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Strength = strength;
                Indicators.Add(indicator);
                indicator.SetUp();

                SwingOscillator[] tmp = new SwingOscillator[cacheSwingOscillator == null ? 1 : cacheSwingOscillator.Length + 1];
                if (cacheSwingOscillator != null)
                    cacheSwingOscillator.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSwingOscillator = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SwingOscillator SwingOscillator(int strength)
        {
            return _indicator.SwingOscillator(Input, strength);
        }

        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SwingOscillator SwingOscillator(Data.IDataSeries input, int strength)
        {
            return _indicator.SwingOscillator(input, strength);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SwingOscillator SwingOscillator(int strength)
        {
            return _indicator.SwingOscillator(Input, strength);
        }

        /// <summary>
        /// Calculate Deviations from recent swing highs and lows to normalize price movement\n\nwww.FreeIndicators.com
        /// </summary>
        /// <returns></returns>
        public Indicator.SwingOscillator SwingOscillator(Data.IDataSeries input, int strength)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SwingOscillator(input, strength);
        }
    }
}
#endregion
